@@ -51,33 +51,20 @@
 namespace aria2 {
 
 DefaultBtMessageReceiver::DefaultBtMessageReceiver():
-  cuid(0),
-  handshakeSent(false),
-  logger(LogFactory::getInstance())
-{
-  if(logger->debug()) {
-    logger->debug("DefaultBtMessageReceiver::instantiated");
-  }
-}
-
-DefaultBtMessageReceiver::~DefaultBtMessageReceiver()
-{
-  if(logger->debug()) {
-    logger->debug("DefaultBtMessageReceiver::deleted");
-  }
-}
+  handshakeSent_(false)
+{}
 
 SharedHandle<BtHandshakeMessage>
 DefaultBtMessageReceiver::receiveHandshake(bool quickReply)
 {
   unsigned char data[BtHandshakeMessage::MESSAGE_LENGTH];
   size_t dataLength = BtHandshakeMessage::MESSAGE_LENGTH;
-  bool retval = peerConnection->receiveHandshake(data, dataLength);
+  bool retval = peerConnection_->receiveHandshake(data, dataLength);
   // To handle tracker's NAT-checking feature
-  if(!handshakeSent && quickReply && dataLength >= 48) {
-    handshakeSent = true;
+  if(!handshakeSent_ && quickReply && dataLength >= 48) {
+    handshakeSent_ = true;
     // check info_hash
-    if(memcmp(bittorrent::getInfoHash(_downloadContext), &data[28],
+    if(memcmp(bittorrent::getInfoHash(downloadContext_), &data[28],
               INFO_HASH_LENGTH) == 0) {
       sendHandshake();
     }
@@ -85,7 +72,8 @@ DefaultBtMessageReceiver::receiveHandshake(bool quickReply)
   if(!retval) {
     return SharedHandle<BtHandshakeMessage>();
   }
-  SharedHandle<BtHandshakeMessage> msg = messageFactory->createHandshakeMessage(data, dataLength);
+  SharedHandle<BtHandshakeMessage> msg =
+    messageFactory_->createHandshakeMessage(data, dataLength);
   msg->validate();
   return msg;
 }
@@ -98,25 +86,25 @@ DefaultBtMessageReceiver::receiveAndSendHandshake()
 
 void DefaultBtMessageReceiver::sendHandshake() {
   SharedHandle<BtMessage> msg =
-    messageFactory->createHandshakeMessage
-    (bittorrent::getInfoHash(_downloadContext), bittorrent::getStaticPeerId());
-  dispatcher->addMessageToQueue(msg);
-  dispatcher->sendMessages();
+    messageFactory_->createHandshakeMessage
+    (bittorrent::getInfoHash(downloadContext_), bittorrent::getStaticPeerId());
+  dispatcher_->addMessageToQueue(msg);
+  dispatcher_->sendMessages();
 }
 
 BtMessageHandle DefaultBtMessageReceiver::receiveMessage() {
   size_t dataLength = 0;
   // Give 0 to PeerConnection::receiveMessage() to prevent memcpy.
-  if(!peerConnection->receiveMessage(0, dataLength)) {
+  if(!peerConnection_->receiveMessage(0, dataLength)) {
     return SharedHandle<BtMessage>();
   }
   BtMessageHandle msg =
-    messageFactory->createBtMessage(peerConnection->getBuffer(), dataLength);
+    messageFactory_->createBtMessage(peerConnection_->getBuffer(), dataLength);
   msg->validate();
   if(msg->getId() == BtPieceMessage::ID) {
     SharedHandle<BtPieceMessage> piecemsg =
-      dynamic_pointer_cast<BtPieceMessage>(msg);
-    piecemsg->setRawMessage(peerConnection->detachBuffer());
+      static_pointer_cast<BtPieceMessage>(msg);
+    piecemsg->setRawMessage(peerConnection_->detachBuffer());
   }
   return msg;
 }
@@ -124,27 +112,25 @@ BtMessageHandle DefaultBtMessageReceiver::receiveMessage() {
 void DefaultBtMessageReceiver::setDownloadContext
 (const SharedHandle<DownloadContext>& downloadContext)
 {
-  _downloadContext = downloadContext;
+  downloadContext_ = downloadContext;
 }
 
-void DefaultBtMessageReceiver::setPeer(const SharedHandle<Peer>& peer)
+void DefaultBtMessageReceiver::setPeerConnection
+(const WeakHandle<PeerConnection>& peerConnection)
 {
-  this->peer = peer;
+  peerConnection_ = peerConnection;
 }
 
-void DefaultBtMessageReceiver::setPeerConnection(const WeakHandle<PeerConnection>& peerConnection)
+void DefaultBtMessageReceiver::setDispatcher
+(const WeakHandle<BtMessageDispatcher>& dispatcher)
 {
-  this->peerConnection = peerConnection;
+  dispatcher_ = dispatcher;
 }
 
-void DefaultBtMessageReceiver::setDispatcher(const WeakHandle<BtMessageDispatcher>& dispatcher)
+void DefaultBtMessageReceiver::setBtMessageFactory
+(const WeakHandle<BtMessageFactory>& factory)
 {
-  this->dispatcher = dispatcher;
-}
-
-void DefaultBtMessageReceiver::setBtMessageFactory(const WeakHandle<BtMessageFactory>& factory)
-{
-  this->messageFactory = factory;
+  messageFactory_ = factory;
 }
 
 } // namespace aria2
