@@ -65,6 +65,7 @@ DefaultPeerStorage::~DefaultPeerStorage()
   delete leecherStateChoke_;
 }
 
+namespace {
 class FindIdenticalPeer {
 private:
   SharedHandle<Peer> peer_;
@@ -77,6 +78,7 @@ public:
        (peer_->getPort() == peer->getPort()));
   }
 };
+}
 
 bool DefaultPeerStorage::isPeerAlreadyAdded(const SharedHandle<Peer>& peer)
 {
@@ -124,17 +126,32 @@ void DefaultPeerStorage::addPeer(const std::vector<SharedHandle<Peer> >& peers)
   }  
 }
 
+void DefaultPeerStorage::addDroppedPeer(const SharedHandle<Peer>& peer)
+{
+  droppedPeers_.push_front(peer);
+  if(droppedPeers_.size() > 50) {
+    droppedPeers_.pop_back();
+  }
+}
+
 const std::deque<SharedHandle<Peer> >& DefaultPeerStorage::getPeers()
 {
   return peers_;
 }
 
+const std::deque<SharedHandle<Peer> >& DefaultPeerStorage::getDroppedPeers()
+{
+  return droppedPeers_;
+}
+
+namespace {
 class FindFinePeer {
 public:
   bool operator()(const SharedHandle<Peer>& peer) const {
     return peer->unused() && peer->isGood();
   }
 };
+}
 
 SharedHandle<Peer> DefaultPeerStorage::getUnusedPeer() {
   std::deque<SharedHandle<Peer> >::const_iterator itr =
@@ -146,6 +163,7 @@ SharedHandle<Peer> DefaultPeerStorage::getUnusedPeer() {
   }
 }
 
+namespace {
 class FindPeer {
 private:
   std::string ipaddr;
@@ -158,6 +176,7 @@ public:
     return ipaddr == peer->getIPAddress() && port == peer->getPort();
   }
 };
+}
 
 SharedHandle<Peer> DefaultPeerStorage::getPeer(const std::string& ipaddr,
                                                uint16_t port) const {
@@ -178,6 +197,7 @@ bool DefaultPeerStorage::isPeerAvailable() {
   return !getUnusedPeer().isNull();
 }
 
+namespace {
 class CollectActivePeer {
 private:
   std::vector<SharedHandle<Peer> >& activePeers_;
@@ -192,6 +212,7 @@ public:
     }
   }
 };
+}
 
 void DefaultPeerStorage::getActivePeers
 (std::vector<SharedHandle<Peer> >& activePeers)
@@ -277,7 +298,7 @@ void DefaultPeerStorage::deleteUnusedPeer(size_t delSize) {
     const SharedHandle<Peer>& p = *itr;
     if(p->unused() && delSize > 0) {
       onErasingPeer(p);
-      delSize--;
+      --delSize;
     } else {
       temp.push_front(p);
     }
@@ -295,6 +316,10 @@ void DefaultPeerStorage::onReturningPeer(const SharedHandle<Peer>& peer)
     removedPeerSessionUploadLength_ += removedStat.getSessionUploadLength();
     cachedTransferStat_ -= removedStat;
 
+    if(peer->isDisconnectedGracefully() && !peer->isIncomingPeer()) {
+      peer->startBadCondition();
+      addDroppedPeer(peer);
+    }
     // Execute choking algorithm if unchoked and interested peer is
     // disconnected.
     if(!peer->amChoking() && peer->peerInterested()) {
