@@ -94,6 +94,15 @@
 # include "BtPostDownloadHandler.h"
 # include "DHTSetup.h"
 # include "DHTRegistry.h"
+# include "DHTNode.h"
+# include "DHTRoutingTable.h"
+# include "DHTTaskQueue.h"
+# include "DHTTaskFactory.h"
+# include "DHTTokenTracker.h"
+# include "DHTMessageDispatcher.h"
+# include "DHTMessageReceiver.h"
+# include "DHTMessageFactory.h"
+# include "DHTMessageCallback.h"
 # include "BtMessageFactory.h"
 # include "BtRequestFactory.h"
 # include "BtMessageDispatcher.h"
@@ -119,6 +128,7 @@ RequestGroup::RequestGroup(const SharedHandle<Option>& option):
   option_(new Option(*option.get())),
   numConcurrentCommand_(option->getAsInt(PREF_SPLIT)),
   numStreamConnection_(0),
+  numStreamCommand_(0),
   numCommand_(0),
   saveControlFile_(true),
   progressInfoFile_(new NullProgressInfoFile()),
@@ -200,6 +210,11 @@ void RequestGroup::closeFile()
 void RequestGroup::createInitialCommand
 (std::vector<Command*>& commands, DownloadEngine* e)
 {
+  // Start session timer here.  When file size becomes known, it will
+  // be reset again in *FileAllocationEntry, because hash check and
+  // file allocation takes a time.  For downloads in which file size
+  // is unknown, session timer will not be reset.
+  downloadContext_->resetDownloadStartTime();
 #ifdef ENABLE_BITTORRENT
   {
     if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
@@ -277,10 +292,20 @@ void RequestGroup::createInitialCommand
                                 SharedHandle<BtProgressInfoFile>
                                 (progressInfoFile))));
       if(metadataGetMode) {
-        if(option_->getAsBool(PREF_ENABLE_DHT)) {
-          std::vector<Command*> dhtCommands;
-          DHTSetup().setup(dhtCommands, e);
-          e->addCommand(dhtCommands);
+        if(option_->getAsBool(PREF_ENABLE_DHT) ||
+           (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+            option_->getAsBool(PREF_ENABLE_DHT6))) {
+          if(option_->getAsBool(PREF_ENABLE_DHT)) {
+            std::vector<Command*> dhtCommands;
+            DHTSetup().setup(dhtCommands, e, AF_INET);
+            e->addCommand(dhtCommands);
+          }
+          if(!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+             option_->getAsBool(PREF_ENABLE_DHT6)) {
+            std::vector<Command*> dhtCommands;
+            DHTSetup().setup(dhtCommands, e, AF_INET6);
+            e->addCommand(dhtCommands);
+          }
         } else {
           logger_->notice("For BitTorrent Magnet URI, enabling DHT is strongly"
                           " recommended. See --enable-dht option.");
@@ -337,16 +362,27 @@ void RequestGroup::createInitialCommand
       }
       progressInfoFile_ = progressInfoFile;
 
-      if(!torrentAttrs->privateTorrent && option_->getAsBool(PREF_ENABLE_DHT)) {
-        std::vector<Command*> dhtCommands;
-        DHTSetup().setup(dhtCommands, e);
-        e->addCommand(dhtCommands);
+      if(!torrentAttrs->privateTorrent &&
+         (option_->getAsBool(PREF_ENABLE_DHT) ||
+          (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+           option_->getAsBool(PREF_ENABLE_DHT6)))) {
+        if(option_->getAsBool(PREF_ENABLE_DHT)) {
+          std::vector<Command*> dhtCommands;
+          DHTSetup().setup(dhtCommands, e, AF_INET);
+          e->addCommand(dhtCommands);
+        }
+        if(!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+           option_->getAsBool(PREF_ENABLE_DHT6)) {
+          std::vector<Command*> dhtCommands;
+          DHTSetup().setup(dhtCommands, e, AF_INET6);
+          e->addCommand(dhtCommands);
+        }
         const std::vector<std::pair<std::string, uint16_t> >& nodes =
           torrentAttrs->nodes;
-        if(!nodes.empty() && DHTSetup::initialized()) {
-          std::vector<std::pair<std::string, uint16_t> > entryPoints(nodes);
+        // TODO Are nodes in torrent IPv4 only?
+        if(!nodes.empty() && DHTRegistry::isInitialized()) {
           DHTEntryPointNameResolveCommand* command =
-            new DHTEntryPointNameResolveCommand(e->newCUID(), e, entryPoints);
+            new DHTEntryPointNameResolveCommand(e->newCUID(), e, nodes);
           command->setTaskQueue(DHTRegistry::getData().taskQueue);
           command->setTaskFactory(DHTRegistry::getData().taskFactory);
           command->setRoutingTable(DHTRegistry::getData().routingTable);
@@ -720,17 +756,17 @@ void RequestGroup::createNextCommand(std::vector<Command*>& commands,
 {
   int numCommand;
   if(getTotalLength() == 0) {
-    if(numStreamConnection_ > 0) {
+    if(numStreamCommand_ > 0) {
       numCommand = 0;
     } else {
       numCommand = 1;
     }
   } else {
-    if(numStreamConnection_ >= numConcurrentCommand_) {
+    if(numStreamCommand_ >= numConcurrentCommand_) {
       numCommand = 0;
     } else {
       numCommand = std::min(downloadContext_->getNumPieces(),
-                            numConcurrentCommand_-numStreamConnection_);
+                            numConcurrentCommand_-numStreamCommand_);
     }
   }
   if(numCommand > 0) {
@@ -825,6 +861,16 @@ void RequestGroup::validateTotalLength(uint64_t actualTotalLength) const
   validateTotalLength(getTotalLength(), actualTotalLength);
 }
 
+void RequestGroup::increaseStreamCommand()
+{
+  ++numStreamCommand_;
+}
+
+void RequestGroup::decreaseStreamCommand()
+{
+  --numStreamCommand_;
+}
+
 void RequestGroup::increaseStreamConnection()
 {
   ++numStreamConnection_;
