@@ -47,7 +47,6 @@
 #endif // HAVE_LIBGNUTLS
 
 #include "message.h"
-#include "a2netcompat.h"
 #include "DlRetryEx.h"
 #include "DlAbortEx.h"
 #include "StringFormat.h"
@@ -55,6 +54,7 @@
 #include "TimeA2.h"
 #include "a2functional.h"
 #include "LogFactory.h"
+#include "A2STR.h"
 #ifdef ENABLE_SSL
 # include "TLSContext.h"
 #endif // ENABLE_SSL
@@ -119,97 +119,59 @@ static const char *errorMsg()
   return errorMsg(SOCKET_ERRNO);
 }
 
-#ifdef HAVE_EPOLL
-SocketCore::PollMethod SocketCore::_pollMethod = SocketCore::POLL_METHOD_EPOLL;
-#else // !HAVE_EPOLL
-SocketCore::PollMethod SocketCore::_pollMethod = SocketCore::POLL_METHOD_SELECT;
-#endif // !HAVE_EPOLL
-
-int SocketCore::_protocolFamily = AF_UNSPEC;
+int SocketCore::protocolFamily_ = AF_UNSPEC;
 
 std::vector<std::pair<struct sockaddr_storage, socklen_t> >
-SocketCore::_bindAddrs;
+SocketCore::bindAddrs_;
 
 #ifdef ENABLE_SSL
-SharedHandle<TLSContext> SocketCore::_tlsContext;
+SharedHandle<TLSContext> SocketCore::tlsContext_;
 
 void SocketCore::setTLSContext(const SharedHandle<TLSContext>& tlsContext)
 {
-  _tlsContext = tlsContext;
+  tlsContext_ = tlsContext;
 }
 #endif // ENABLE_SSL
 
-SocketCore::SocketCore(int sockType):_sockType(sockType), sockfd(-1)  {
+SocketCore::SocketCore(int sockType):sockType_(sockType), sockfd_(-1)  {
   init();
 }
 
-SocketCore::SocketCore(sock_t sockfd, int sockType):_sockType(sockType), sockfd(sockfd) {
+SocketCore::SocketCore(sock_t sockfd, int sockType):sockType_(sockType), sockfd_(sockfd) {
   init();
 }
 
 void SocketCore::init()
 {
+  blocking_ = true;
+  secure_ = 0;
 
-#ifdef HAVE_EPOLL
-
-  _epfd = -1;
-
-#endif // HAVE_EPOLL
-
-  blocking = true;
-  secure = 0;
-
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
 
 #ifdef HAVE_LIBSSL
   // for SSL
   ssl = NULL;
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-  sslSession = NULL;
-  peekBufMax = 4096;
-  peekBuf = 0;
-  peekBufLength = 0;
+  sslSession_ = 0;
+  peekBufMax_ = 4096;
+  peekBuf_ = 0;
+  peekBufLength_ = 0;
 #endif //HAVE_LIBGNUTLS
 }
 
 SocketCore::~SocketCore() {
   closeConnection();
-
-#ifdef HAVE_EPOLL
-
-  if(_epfd != -1) {
-    CLOSE(_epfd);
-  }
-
-#endif // HAVE_EPOLL
-
 #ifdef HAVE_LIBGNUTLS
-  delete [] peekBuf;
+  delete [] peekBuf_;
 #endif // HAVE_LIBGNUTLS
 }
 
-template<typename T>
-std::string uitos(T value)
-{
-  std::string str;
-  if(value == 0) {
-    str = "0";
-    return str;
-  }
-  while(value) {
-    char digit = value%10+'0';
-    str.insert(str.begin(), digit);
-    value /= 10;
-  }
-  return str;
-}
-
 void SocketCore::create(int family, int protocol)
 {
   closeConnection();
-  sock_t fd = socket(family, _sockType, protocol);
+  sock_t fd = socket(family, sockType_, protocol);
   if(fd == (sock_t) -1) {
     throw DL_ABORT_EX
       (StringFormat("Failed to create socket. Cause:%s", errorMsg()).str());
@@ -221,7 +183,7 @@ void SocketCore::create(int family, int protocol)
     throw DL_ABORT_EX
       (StringFormat("Failed to create socket. Cause:%s", errorMsg()).str());
   }
-  sockfd = fd;
+  sockfd_ = fd;
 }
 
 static sock_t bindInternal(int family, int socktype, int protocol,
@@ -238,6 +200,14 @@ static sock_t bindInternal(int family, int socktype, int protocol,
     CLOSE(fd);
     return -1;
   }
+  if(family == AF_INET6) {
+    int sockopt = 1;
+    if(setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (a2_sockopt_t) &sockopt,
+                  sizeof(sockopt)) < 0) {
+      CLOSE(fd);
+      return -1;
+    }
+  }
   if(::bind(fd, addr, addrlen) == -1) {
     error = errorMsg();
     CLOSE(fd);
@@ -251,8 +221,8 @@ static sock_t bindTo
  int getaddrinfoFlags, std::string& error)
 {
   struct addrinfo* res;
-  int s = callGetaddrinfo(&res, host, uitos(port).c_str(), family, sockType,
-                          getaddrinfoFlags, 0);  
+  int s = callGetaddrinfo(&res, host, util::uitos(port).c_str(),
+                          family, sockType, getaddrinfoFlags, 0);  
   if(s) {
     error = gai_strerror(s);
     return -1;
@@ -273,69 +243,72 @@ void SocketCore::bindWithFamily(uint16_t port, int family, int flags)
 {
   closeConnection();
   std::string error;
-  sock_t fd = bindTo(0, port, family, _sockType, flags, error);
+  sock_t fd = bindTo(0, port, family, sockType_, flags, error);
   if(fd == (sock_t) -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
   } else {
-    sockfd = fd;
+    sockfd_ = fd;
   }
 }
 
-void SocketCore::bind(const std::string& addr, uint16_t port, int flags)
+void SocketCore::bind
+(const std::string& addr, uint16_t port, int family, int flags)
 {
   closeConnection();
   std::string error;
-  sock_t fd =
-    bindTo(addr.c_str(), port, _protocolFamily, _sockType, flags, error);
-  if(fd == (sock_t)-1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
+  const char* addrp;
+  if(addr.empty()) {
+    addrp = 0;
   } else {
-    sockfd = fd;
+    addrp = addr.c_str();
   }
-}
-
-void SocketCore::bind(uint16_t port, int flags)
-{
-  closeConnection();
-  std::string error;
-  if(!(flags&AI_PASSIVE) || _bindAddrs.empty()) {
-    sock_t fd = bindTo(0, port, _protocolFamily, _sockType, flags, error);
+  if(!(flags&AI_PASSIVE) || bindAddrs_.empty()) {
+    sock_t fd = bindTo(addrp, port, family, sockType_, flags, error);
     if(fd != (sock_t) -1) {
-      sockfd = fd;
+      sockfd_ = fd;
     }
   } else {
     for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
-          const_iterator i = _bindAddrs.begin(), eoi = _bindAddrs.end();
+          const_iterator i = bindAddrs_.begin(), eoi = bindAddrs_.end();
         i != eoi; ++i) {
       char host[NI_MAXHOST];
       int s;
       s = getnameinfo(reinterpret_cast<const struct sockaddr*>(&(*i).first),
                       (*i).second,
-                      host, NI_MAXHOST, 0, NI_MAXSERV,
+                      host, NI_MAXHOST, 0, 0,
                       NI_NUMERICHOST);
       if(s) {
         error = gai_strerror(s);
         continue;
       }
-      sock_t fd = bindTo(host, port, _protocolFamily, _sockType, flags, error);
+      if(addrp && strcmp(host, addrp) != 0) {
+        error = "Given address and resolved address do not match.";
+        continue;
+      }
+      sock_t fd = bindTo(addrp, port, family, sockType_, flags, error);
       if(fd != (sock_t)-1) {
-        sockfd = fd;
+        sockfd_ = fd;
         break;
       }
     }
   }
-  if(sockfd == (sock_t) -1) {
+  if(sockfd_ == (sock_t) -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
   }
 }
 
+void SocketCore::bind(uint16_t port, int flags)
+{
+  bind(A2STR::NIL, port, protocolFamily_, flags);
+}
+
 void SocketCore::bind(const struct sockaddr* addr, socklen_t addrlen)
 {
   closeConnection();
   std::string error;
-  sock_t fd = bindInternal(addr->sa_family, _sockType, 0, addr, addrlen, error);
+  sock_t fd = bindInternal(addr->sa_family, sockType_, 0, addr, addrlen, error);
   if(fd != (sock_t)-1) {
-    sockfd = fd;
+    sockfd_ = fd;
   } else {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
   }
@@ -343,7 +316,7 @@ void SocketCore::bind(const struct sockaddr* addr, socklen_t addrlen)
 
 void SocketCore::beginListen()
 {
-  if(listen(sockfd, 1) == -1) {
+  if(listen(sockfd_, 1) == -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_LISTEN, errorMsg()).str());
   }
 }
@@ -353,22 +326,37 @@ SocketCore* SocketCore::acceptConnection() const
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
   sock_t fd;
-  while((fd = accept(sockfd, reinterpret_cast<struct sockaddr*>(&sockaddr), &len)) == (sock_t) -1 && SOCKET_ERRNO == A2_EINTR);
+  while((fd = accept(sockfd_, reinterpret_cast<struct sockaddr*>(&sockaddr), &len)) == (sock_t) -1 && SOCKET_ERRNO == A2_EINTR);
   if(fd == (sock_t) -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_ACCEPT, errorMsg()).str());
   }
-  return new SocketCore(fd, _sockType);
+  return new SocketCore(fd, sockType_);
 }
 
 void SocketCore::getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const
 {
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
+  getAddrInfo(sockaddr, len);
+  addrinfo = util::getNumericNameInfo
+    (reinterpret_cast<const struct sockaddr*>(&sockaddr), len);
+}
+
+void SocketCore::getAddrInfo
+(struct sockaddr_storage& sockaddr, socklen_t& len) const
+{
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
-  if(getsockname(sockfd, addrp, &len) == -1) {
+  if(getsockname(sockfd_, addrp, &len) == -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
   }
-  addrinfo = util::getNumericNameInfo(addrp, len);
+}
+
+int SocketCore::getAddressFamily() const
+{
+  struct sockaddr_storage sockaddr;
+  socklen_t len = sizeof(sockaddr);
+  getAddrInfo(sockaddr, len);
+  return sockaddr.ss_family;
 }
 
 void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
@@ -376,7 +364,7 @@ void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
-  if(getpeername(sockfd, addrp, &len) == -1) {
+  if(getpeername(sockfd_, addrp, &len) == -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
   }
   peerinfo = util::getNumericNameInfo(addrp, len);
@@ -388,8 +376,8 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
   std::string error;
   struct addrinfo* res;
   int s;
-  s = callGetaddrinfo(&res, host.c_str(), uitos(port).c_str(), _protocolFamily,
-                      _sockType, 0, 0);
+  s = callGetaddrinfo(&res, host.c_str(), util::uitos(port).c_str(),
+                      protocolFamily_, sockType_, 0, 0);
   if(s) {
     throw DL_ABORT_EX(StringFormat(EX_RESOLVE_HOSTNAME,
                                    host.c_str(), gai_strerror(s)).str());
@@ -408,10 +396,10 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
       CLOSE(fd);
       continue;
     }
-    if(!_bindAddrs.empty()) {
+    if(!bindAddrs_.empty()) {
       bool bindSuccess = false;
       for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
-            const_iterator i = _bindAddrs.begin(), eoi = _bindAddrs.end();
+            const_iterator i = bindAddrs_.begin(), eoi = bindAddrs_.end();
           i != eoi; ++i) {
         if(::bind(fd,reinterpret_cast<const struct sockaddr*>(&(*i).first),
                   (*i).second) == -1) {
@@ -430,21 +418,21 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
       }
     }
 
-    sockfd = fd;
+    sockfd_ = fd;
     // make socket non-blocking mode
     setNonBlockingMode();
     if(connect(fd, rp->ai_addr, rp->ai_addrlen) == -1 &&
        SOCKET_ERRNO != A2_EINPROGRESS) {
       error = errorMsg();
-      CLOSE(sockfd);
-      sockfd = (sock_t) -1;
+      CLOSE(sockfd_);
+      sockfd_ = (sock_t) -1;
       continue;
     }
     // TODO at this point, connection may not be established and it may fail
     // later. In such case, next ai_addr should be tried.
     break;
   }
-  if(sockfd == (sock_t) -1) {
+  if(sockfd_ == (sock_t) -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_CONNECT, host.c_str(),
                                    error.c_str()).str());
   }
@@ -453,7 +441,7 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
 void SocketCore::setSockOpt
 (int level, int optname, void* optval, socklen_t optlen)
 {
-  if(setsockopt(sockfd, level, optname, (a2_sockopt_t)optval, optlen) < 0) {
+  if(setsockopt(sockfd_, level, optname, (a2_sockopt_t)optval, optlen) < 0) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_SET_OPT, errorMsg()).str());
   }
 }   
@@ -511,180 +499,166 @@ void SocketCore::setNonBlockingMode()
 {
 #ifdef __MINGW32__
   static u_long flag = 1;
-  if (::ioctlsocket(sockfd, FIONBIO, &flag) == -1) {
+  if (::ioctlsocket(sockfd_, FIONBIO, &flag) == -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_NONBLOCKING, errorMsg()).str());
   }
 #else
   int flags;
-  while((flags = fcntl(sockfd, F_GETFL, 0)) == -1 && errno == EINTR);
+  while((flags = fcntl(sockfd_, F_GETFL, 0)) == -1 && errno == EINTR);
   // TODO add error handling
-  while(fcntl(sockfd, F_SETFL, flags|O_NONBLOCK) == -1 && errno == EINTR);
+  while(fcntl(sockfd_, F_SETFL, flags|O_NONBLOCK) == -1 && errno == EINTR);
 #endif // __MINGW32__
-  blocking = false;
+  blocking_ = false;
 }
 
 void SocketCore::setBlockingMode()
 {
 #ifdef __MINGW32__
   static u_long flag = 0;
-  if (::ioctlsocket(sockfd, FIONBIO, &flag) == -1) {
+  if (::ioctlsocket(sockfd_, FIONBIO, &flag) == -1) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_BLOCKING, errorMsg()).str());
   }
 #else
   int flags;
-  while((flags = fcntl(sockfd, F_GETFL, 0)) == -1 && errno == EINTR);
+  while((flags = fcntl(sockfd_, F_GETFL, 0)) == -1 && errno == EINTR);
   // TODO add error handling
-  while(fcntl(sockfd, F_SETFL, flags&(~O_NONBLOCK)) == -1 && errno == EINTR);
+  while(fcntl(sockfd_, F_SETFL, flags&(~O_NONBLOCK)) == -1 && errno == EINTR);
 #endif // __MINGW32__
-  blocking = true;
+  blocking_ = true;
 }
 
 void SocketCore::closeConnection()
 {
 #ifdef HAVE_LIBSSL
   // for SSL
-  if(secure) {
+  if(secure_) {
     SSL_shutdown(ssl);
   }
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-  if(secure) {
-    gnutls_bye(sslSession, GNUTLS_SHUT_RDWR);
+  if(secure_) {
+    gnutls_bye(sslSession_, GNUTLS_SHUT_RDWR);
   }
 #endif // HAVE_LIBGNUTLS
-  if(sockfd != (sock_t) -1) {
-    CLOSE(sockfd);
-    sockfd = -1;
+  if(sockfd_ != (sock_t) -1) {
+    CLOSE(sockfd_);
+    sockfd_ = -1;
   }
 #ifdef HAVE_LIBSSL
   // for SSL
-  if(secure) {
+  if(secure_) {
     SSL_free(ssl);
   }
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-  if(secure) {
-    gnutls_deinit(sslSession);
+  if(secure_) {
+    gnutls_deinit(sslSession_);
   }
 #endif // HAVE_LIBGNUTLS
 }
 
-#ifdef HAVE_EPOLL
-
-void SocketCore::initEPOLL()
-{
-  if((_epfd = epoll_create(1)) == -1) {
-    throw DL_RETRY_EX(StringFormat("epoll_create failed:%s", errorMsg()).str());
+#ifndef __MINGW32__
+# define CHECK_FD(fd)                                                   \
+  if(fd < 0 || FD_SETSIZE <= fd) {                                      \
+    logger_->warn("Detected file descriptor >= FD_SETSIZE or < 0. "     \
+                  "Download may slow down or fail.");                   \
+    return false;                                                       \
   }
-
-  memset(&_epEvent, 0, sizeof(struct epoll_event));
-  _epEvent.events = EPOLLIN|EPOLLOUT;
-  _epEvent.data.fd = sockfd;
-
-  if(epoll_ctl(_epfd, EPOLL_CTL_ADD, sockfd, &_epEvent) == -1) {
-    throw DL_RETRY_EX(StringFormat("epoll_ctl failed:%s", errorMsg()).str());
-  }
-}
-
-#endif // HAVE_EPOLL
+#endif // !__MINGW32__
 
 bool SocketCore::isWritable(time_t timeout)
 {
-#ifdef HAVE_EPOLL
-  if(_pollMethod == SocketCore::POLL_METHOD_EPOLL) {
-    if(_epfd == -1) {
-      initEPOLL();
-    }
-    struct epoll_event epEvents[1];
-    int r;
-    while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
-    if(r > 0) {
-      return epEvents[0].events&(EPOLLOUT|EPOLLHUP|EPOLLERR);
-    } else if(r == 0) {
+#ifdef HAVE_POLL
+  struct pollfd p;
+  p.fd = sockfd_;
+  p.events = POLLOUT;
+  int r;
+  while((r = poll(&p, 1, timeout*1000)) == -1 && errno == EINTR);
+  if(r > 0) {
+    return p.revents&(POLLOUT|POLLHUP|POLLERR);
+  } else if(r == 0) {
+    return false;
+  } else {
+    throw DL_RETRY_EX
+      (StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
+  }
+#else // !HAVE_POLL
+# ifndef __MINGW32__
+  CHECK_FD(sockfd_);
+# endif // !__MINGW32__
+  fd_set fds;
+  FD_ZERO(&fds);
+  FD_SET(sockfd_, &fds);
+
+  struct timeval tv;
+  tv.tv_sec = timeout;
+  tv.tv_usec = 0;
+
+  int r = select(sockfd_+1, NULL, &fds, NULL, &tv);
+  if(r == 1) {
+    return true;
+  } else if(r == 0) {
+    // time out
+    return false;
+  } else {
+    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
       return false;
     } else {
-      throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
-    }
-  } else
-#endif // HAVE_EPOLL
-    if(_pollMethod == SocketCore::POLL_METHOD_SELECT) {
-      fd_set fds;
-      FD_ZERO(&fds);
-      FD_SET(sockfd, &fds);
-
-      struct timeval tv;
-      tv.tv_sec = timeout;
-      tv.tv_usec = 0;
-
-      int r = select(sockfd+1, NULL, &fds, NULL, &tv);
-      if(r == 1) {
-        return true;
-      } else if(r == 0) {
-        // time out
-        return false;
-      } else {
-        if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
-          return false;
-        } else {
-          throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
-        }
-      }
-    } else {
-      abort();
+      throw DL_RETRY_EX
+        (StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
     }
+  }
+#endif // !HAVE_POLL
 }
 
 bool SocketCore::isReadable(time_t timeout)
 {
 #ifdef HAVE_LIBGNUTLS
-  if(secure && peekBufLength > 0) {
+  if(secure_ && peekBufLength_ > 0) {
     return true;
   }
 #endif // HAVE_LIBGNUTLS
+#ifdef HAVE_POLL
+  struct pollfd p;
+  p.fd = sockfd_;
+  p.events = POLLIN;
+  int r;
+  while((r = poll(&p, 1, timeout*1000)) == -1 && errno == EINTR);
+  if(r > 0) {
+    return p.revents&(POLLIN|POLLHUP|POLLERR);
+  } else if(r == 0) {
+    return false;
+  } else {
+    throw DL_RETRY_EX
+      (StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
+  }
+#else // !HAVE_POLL
+# ifndef __MINGW32__
+  CHECK_FD(sockfd_);
+# endif // !__MINGW32__
+  fd_set fds;
+  FD_ZERO(&fds);
+  FD_SET(sockfd_, &fds);
 
-#ifdef HAVE_EPOLL
-  if(_pollMethod == SocketCore::POLL_METHOD_EPOLL) {
-    if(_epfd == -1) {
-      initEPOLL();
-    }
-    struct epoll_event epEvents[1];
-    int r;
-    while((r = epoll_wait(_epfd, epEvents, 1, 0)) == -1 && errno == EINTR);
+  struct timeval tv;
+  tv.tv_sec = timeout;
+  tv.tv_usec = 0;
 
-    if(r > 0) {
-      return epEvents[0].events&(EPOLLIN|EPOLLHUP|EPOLLERR);
-    } else if(r == 0) {
+  int r = select(sockfd_+1, &fds, NULL, NULL, &tv);
+  if(r == 1) {
+    return true;
+  } else if(r == 0) {
+    // time out
+    return false;
+  } else {
+    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
       return false;
     } else {
-      throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
-    }
-  } else
-#endif // HAVE_EPOLL
-    if(_pollMethod == SocketCore::POLL_METHOD_SELECT) {
-      fd_set fds;
-      FD_ZERO(&fds);
-      FD_SET(sockfd, &fds);
-
-      struct timeval tv;
-      tv.tv_sec = timeout;
-      tv.tv_usec = 0;
-
-      int r = select(sockfd+1, &fds, NULL, NULL, &tv);
-      if(r == 1) {
-        return true;
-      } else if(r == 0) {
-        // time out
-        return false;
-      } else {
-        if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
-          return false;
-        } else {
-          throw DL_RETRY_EX(StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
-        }
-      }
-    } else {
-      abort();
+      throw DL_RETRY_EX
+        (StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
     }
+  }
+#endif // !HAVE_POLL
 }
 
 #ifdef HAVE_LIBSSL
@@ -694,9 +668,9 @@ int SocketCore::sslHandleEAGAIN(int ret)
   if(error == SSL_ERROR_WANT_READ || error == SSL_ERROR_WANT_WRITE) {
     ret = 0;
     if(error == SSL_ERROR_WANT_READ) {
-      _wantRead = true;
+      wantRead_ = true;
     } else {
-      _wantWrite = true;
+      wantWrite_ = true;
     }
   }
   return ret;
@@ -706,11 +680,11 @@ int SocketCore::sslHandleEAGAIN(int ret)
 #ifdef HAVE_LIBGNUTLS
 void SocketCore::gnutlsRecordCheckDirection()
 {
-  int direction = gnutls_record_get_direction(sslSession);
+  int direction = gnutls_record_get_direction(sslSession_);
   if(direction == 0) {
-    _wantRead = true;
+    wantRead_ = true;
   } else { // if(direction == 1) {
-    _wantWrite = true;
+    wantWrite_ = true;
   }
 }
 #endif // HAVE_LIBGNUTLS
@@ -718,14 +692,14 @@ void SocketCore::gnutlsRecordCheckDirection()
 ssize_t SocketCore::writeData(const char* data, size_t len)
 {
   ssize_t ret = 0;
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
 
-  if(!secure) {
-    while((ret = send(sockfd, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
+  if(!secure_) {
+    while((ret = send(sockfd_, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
     if(ret == -1) {
       if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
-        _wantWrite = true;
+        wantWrite_ = true;
         ret = 0;
       } else {
         throw DL_RETRY_EX(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
@@ -744,7 +718,7 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
     }
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-    while((ret = gnutls_record_send(sslSession, data, len)) ==
+    while((ret = gnutls_record_send(sslSession_, data, len)) ==
           GNUTLS_E_INTERRUPTED);
     if(ret == GNUTLS_E_AGAIN) {
       gnutlsRecordCheckDirection();
@@ -761,15 +735,15 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
 void SocketCore::readData(char* data, size_t& len)
 {
   ssize_t ret = 0;
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
 
-  if(!secure) {    
-    while((ret = recv(sockfd, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
+  if(!secure_) {    
+    while((ret = recv(sockfd_, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
     
     if(ret == -1) {
       if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
-        _wantRead = true;
+        wantRead_ = true;
         ret = 0;
       } else {
         throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
@@ -807,15 +781,15 @@ void SocketCore::readData(char* data, size_t& len)
 void SocketCore::peekData(char* data, size_t& len)
 {
   ssize_t ret = 0;
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
 
-  if(!secure) {
-    while((ret = recv(sockfd, data, len, MSG_PEEK)) == -1 &&
+  if(!secure_) {
+    while((ret = recv(sockfd_, data, len, MSG_PEEK)) == -1 &&
           SOCKET_ERRNO == A2_EINTR);
     if(ret == -1) {
       if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
-        _wantRead = true;
+        wantRead_ = true;
         ret = 0;
       } else {
         throw DL_RETRY_EX(StringFormat(EX_SOCKET_PEEK, errorMsg()).str());
@@ -853,18 +827,15 @@ void SocketCore::peekData(char* data, size_t& len)
 #ifdef HAVE_LIBGNUTLS
 size_t SocketCore::shiftPeekData(char* data, size_t len)
 {
-  if(peekBufLength <= len) {
-    memcpy(data, peekBuf, peekBufLength);
-    size_t ret = peekBufLength;
-    peekBufLength = 0;
+  if(peekBufLength_ <= len) {
+    memcpy(data, peekBuf_, peekBufLength_);
+    size_t ret = peekBufLength_;
+    peekBufLength_ = 0;
     return ret;
   } else {
-    memcpy(data, peekBuf, len);
-    char* temp = new char[peekBufMax];
-    memcpy(temp, peekBuf+len, peekBufLength-len);
-    delete [] peekBuf;
-    peekBuf = temp;
-    peekBufLength -= len;
+    memcpy(data, peekBuf_, len);
+    peekBufLength_ -= len;    
+    memmove(peekBuf_, peekBuf_+len, peekBufLength_);
     return len;
   }
 
@@ -872,15 +843,15 @@ size_t SocketCore::shiftPeekData(char* data, size_t len)
 
 void SocketCore::addPeekData(char* data, size_t len)
 {
-  if(peekBufLength+len > peekBufMax) {
-    char* temp = new char[peekBufMax+len];
-    memcpy(temp, peekBuf, peekBufLength);
-    delete [] peekBuf;
-    peekBuf = temp;
-    peekBufMax = peekBufLength+len;
+  if(peekBufLength_+len > peekBufMax_) {
+    char* temp = new char[peekBufMax_+len];
+    memcpy(temp, peekBuf_, peekBufLength_);
+    delete [] peekBuf_;
+    peekBuf_ = temp;
+    peekBufMax_ = peekBufLength_+len;
   }
-  memcpy(peekBuf+peekBufLength, data, len);
-  peekBufLength += len;
+  memcpy(peekBuf_+peekBufLength_, data, len);
+  peekBufLength_ += len;
 }
 
 static ssize_t GNUTLS_RECORD_RECV_NO_INTERRUPT
@@ -901,7 +872,7 @@ ssize_t SocketCore::gnutlsRecv(char* data, size_t len)
   size_t plen = shiftPeekData(data, len);
   if(plen < len) {
     ssize_t ret = GNUTLS_RECORD_RECV_NO_INTERRUPT
-      (sslSession, data+plen, len-plen);
+      (sslSession_, data+plen, len-plen);
     if(ret == GNUTLS_E_AGAIN) {
       return GNUTLS_E_AGAIN;
     }
@@ -913,34 +884,34 @@ ssize_t SocketCore::gnutlsRecv(char* data, size_t len)
 
 ssize_t SocketCore::gnutlsPeek(char* data, size_t len)
 {
-  if(peekBufLength >= len) {
-    memcpy(data, peekBuf, len);
+  if(peekBufLength_ >= len) {
+    memcpy(data, peekBuf_, len);
     return len;
   } else {
-    memcpy(data, peekBuf, peekBufLength);
+    memcpy(data, peekBuf_, peekBufLength_);
     ssize_t ret = GNUTLS_RECORD_RECV_NO_INTERRUPT
-      (sslSession, data+peekBufLength, len-peekBufLength);
+      (sslSession_, data+peekBufLength_, len-peekBufLength_);
     if(ret == GNUTLS_E_AGAIN) {
       return GNUTLS_E_AGAIN;
     }
-    addPeekData(data+peekBufLength, ret);
-    return peekBufLength;
+    addPeekData(data+peekBufLength_, ret);
+    return peekBufLength_;
   }
 }
 #endif // HAVE_LIBGNUTLS
 
 void SocketCore::prepareSecureConnection()
 {
-  if(!secure) {
+  if(!secure_) {
 #ifdef HAVE_LIBSSL
     // for SSL
-    ssl = SSL_new(_tlsContext->getSSLCtx());
+    ssl = SSL_new(tlsContext_->getSSLCtx());
     if(!ssl) {
       throw DL_ABORT_EX
         (StringFormat(EX_SSL_INIT_FAILURE,
                       ERR_error_string(ERR_get_error(), 0)).str());
     }
-    if(SSL_set_fd(ssl, sockfd) == 0) {
+    if(SSL_set_fd(ssl, sockfd_) == 0) {
       throw DL_ABORT_EX
         (StringFormat(EX_SSL_INIT_FAILURE,
                       ERR_error_string(ERR_get_error(), 0)).str());
@@ -948,31 +919,31 @@ void SocketCore::prepareSecureConnection()
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
     int r;
-    gnutls_init(&sslSession, GNUTLS_CLIENT);
+    gnutls_init(&sslSession_, GNUTLS_CLIENT);
     // It seems err is not error message, but the argument string
     // which causes syntax error.
     const char* err;
     // Disables TLS1.1 here because there are servers that don't
     // understand TLS1.1.
-    r = gnutls_priority_set_direct(sslSession, "NORMAL:!VERS-TLS1.1", &err);
+    r = gnutls_priority_set_direct(sslSession_, "NORMAL:!VERS-TLS1.1", &err);
     if(r != GNUTLS_E_SUCCESS) {
       throw DL_ABORT_EX
         (StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(r)).str());
     }
     // put the x509 credentials to the current session
-    gnutls_credentials_set(sslSession, GNUTLS_CRD_CERTIFICATE,
-                           _tlsContext->getCertCred());
-    gnutls_transport_set_ptr(sslSession, (gnutls_transport_ptr_t)sockfd);
+    gnutls_credentials_set(sslSession_, GNUTLS_CRD_CERTIFICATE,
+                           tlsContext_->getCertCred());
+    gnutls_transport_set_ptr(sslSession_, (gnutls_transport_ptr_t)sockfd_);
 #endif // HAVE_LIBGNUTLS
-    secure = 1;
+    secure_ = 1;
   }
 }
 
 bool SocketCore::initiateSecureConnection(const std::string& hostname)
 {
-  if(secure == 1) {
-    _wantRead = false;
-    _wantWrite = false;
+  if(secure_ == 1) {
+    wantRead_ = false;
+    wantWrite_ = false;
 #ifdef HAVE_LIBSSL
     int e = SSL_connect(ssl);
 
@@ -983,14 +954,14 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         break;
 
       case SSL_ERROR_WANT_READ:
-        _wantRead = true;
+        wantRead_ = true;
         return false;
       case SSL_ERROR_WANT_WRITE:
-        _wantWrite = true;
+        wantWrite_ = true;
         return false;
       case SSL_ERROR_WANT_X509_LOOKUP:
       case SSL_ERROR_ZERO_RETURN:
-        if (blocking) {
+        if (blocking_) {
           throw DL_ABORT_EX
             (StringFormat(EX_SSL_CONNECT_ERROR, ssl_error).str());
         }
@@ -1007,7 +978,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
           (StringFormat(EX_SSL_UNKNOWN_ERROR, ssl_error).str());
       }
     }
-    if(_tlsContext->peerVerificationEnabled()) {
+    if(tlsContext_->peerVerificationEnabled()) {
       // verify peer
       X509* peerCert = SSL_get_peer_certificate(ssl);
       if(!peerCert) {
@@ -1052,7 +1023,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
     }
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
-    int ret = gnutls_handshake(sslSession);
+    int ret = gnutls_handshake(sslSession_);
     if(ret == GNUTLS_E_AGAIN) {
       gnutlsRecordCheckDirection();
       return false;
@@ -1061,10 +1032,10 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         (StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(ret)).str());
     }
 
-    if(_tlsContext->peerVerificationEnabled()) {
+    if(tlsContext_->peerVerificationEnabled()) {
       // verify peer
       unsigned int status;
-      ret = gnutls_certificate_verify_peers2(sslSession, &status);
+      ret = gnutls_certificate_verify_peers2(sslSession_, &status);
       if(ret < 0) {
         throw DL_ABORT_EX
           (StringFormat("gnutls_certificate_verify_peer2() failed. Cause: %s",
@@ -1087,13 +1058,13 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         }
       }
       // certificate type: only X509 is allowed.
-      if(gnutls_certificate_type_get(sslSession) != GNUTLS_CRT_X509) {
+      if(gnutls_certificate_type_get(sslSession_) != GNUTLS_CRT_X509) {
         throw DL_ABORT_EX("Certificate type is not X509.");
       }
 
       unsigned int peerCertsLength;
       const gnutls_datum_t* peerCerts = gnutls_certificate_get_peers
-        (sslSession, &peerCertsLength);
+        (sslSession_, &peerCertsLength);
       if(!peerCerts) {
         throw DL_ABORT_EX(MSG_NO_CERT_FOUND);
       }
@@ -1135,9 +1106,9 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         }
       }
     }
-    peekBuf = new char[peekBufMax];
+    peekBuf_ = new char[peekBufMax_];
 #endif // HAVE_LIBGNUTLS
-    secure = 2;
+    secure_ = 2;
     return true;
   } else {
     return true;
@@ -1147,13 +1118,13 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
 ssize_t SocketCore::writeData(const char* data, size_t len,
                               const std::string& host, uint16_t port)
 {
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
 
   struct addrinfo* res;
   int s;
-  s = callGetaddrinfo(&res, host.c_str(), uitos(port).c_str(), _protocolFamily,
-                      _sockType, 0, 0);
+  s = callGetaddrinfo(&res, host.c_str(), util::uitos(port).c_str(),
+                      protocolFamily_, sockType_, 0, 0);
   if(s) {
     throw DL_ABORT_EX(StringFormat(EX_SOCKET_SEND, gai_strerror(s)).str());
   }
@@ -1161,13 +1132,13 @@ ssize_t SocketCore::writeData(const char* data, size_t len,
   struct addrinfo* rp;
   ssize_t r = -1;
   for(rp = res; rp; rp = rp->ai_next) {
-    while((r = sendto(sockfd, data, len, 0, rp->ai_addr, rp->ai_addrlen)) == -1
+    while((r = sendto(sockfd_, data, len, 0, rp->ai_addr, rp->ai_addrlen)) == -1
           && A2_EINTR == SOCKET_ERRNO);
     if(r == static_cast<ssize_t>(len)) {
       break;
     }
     if(r == -1 && A2_WOULDBLOCK(SOCKET_ERRNO)) {
-      _wantWrite = true;
+      wantWrite_ = true;
       r = 0;
       break;
     }
@@ -1182,17 +1153,17 @@ ssize_t SocketCore::readDataFrom(char* data, size_t len,
                                  std::pair<std::string /* numerichost */,
                                  uint16_t /* port */>& sender)
 {
-  _wantRead = false;
-  _wantWrite = false;
+  wantRead_ = false;
+  wantWrite_ = false;
   struct sockaddr_storage sockaddr;
   socklen_t sockaddrlen = sizeof(struct sockaddr_storage);
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
   ssize_t r;
-  while((r = recvfrom(sockfd, data, len, 0, addrp, &sockaddrlen)) == -1 &&
+  while((r = recvfrom(sockfd_, data, len, 0, addrp, &sockaddrlen)) == -1 &&
         A2_EINTR == SOCKET_ERRNO);
   if(r == -1) {
     if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
-      _wantRead = true;
+      wantRead_ = true;
       r = 0;
     } else {
       throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
@@ -1209,7 +1180,7 @@ std::string SocketCore::getSocketError() const
   int error;
   socklen_t optlen = sizeof(error);
 
-  if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (a2_sockopt_t) &error, &optlen) == -1) {
+  if(getsockopt(sockfd_, SOL_SOCKET, SO_ERROR, (a2_sockopt_t) &error, &optlen) == -1) {
     throw DL_ABORT_EX(StringFormat("Failed to get socket error: %s",
                                    errorMsg()).str());
   }
@@ -1222,44 +1193,32 @@ std::string SocketCore::getSocketError() const
 
 bool SocketCore::wantRead() const
 {
-  return _wantRead;
+  return wantRead_;
 }
 
 bool SocketCore::wantWrite() const
 {
-  return _wantWrite;
-}
-
-#ifdef HAVE_EPOLL
-void SocketCore::useEpoll()
-{
-  _pollMethod = SocketCore::POLL_METHOD_EPOLL;
-}
-#endif // HAVE_EPOLL
-
-void SocketCore::useSelect()
-{
-  _pollMethod = SocketCore::POLL_METHOD_SELECT;
+  return wantWrite_;
 }
 
 void SocketCore::bindAddress(const std::string& iface)
 {
   std::vector<std::pair<struct sockaddr_storage, socklen_t> > bindAddrs;
-  getInterfaceAddress(bindAddrs, iface, _protocolFamily);
+  getInterfaceAddress(bindAddrs, iface, protocolFamily_);
   if(bindAddrs.empty()) {
     throw DL_ABORT_EX
       (StringFormat(MSG_INTERFACE_NOT_FOUND,
                     iface.c_str(), "not available").str());
   } else {
-    _bindAddrs = bindAddrs;
+    bindAddrs_ = bindAddrs;
     for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
-          const_iterator i = _bindAddrs.begin(), eoi = _bindAddrs.end();
+          const_iterator i = bindAddrs_.begin(), eoi = bindAddrs_.end();
         i != eoi; ++i) {
       char host[NI_MAXHOST];
       int s;
       s = getnameinfo(reinterpret_cast<const struct sockaddr*>(&(*i).first),
                       (*i).second,
-                      host, NI_MAXHOST, 0, NI_MAXSERV,
+                      host, NI_MAXHOST, 0, 0,
                       NI_NUMERICHOST);
       if(s == 0) {
         if(LogFactory::getInstance()->debug()) {
