@@ -44,7 +44,7 @@ public:
 
 CPPUNIT_TEST_SUITE_REGISTRATION( MultiDiskAdaptorTest );
 
-std::deque<SharedHandle<FileEntry> > createEntries() {
+std::vector<SharedHandle<FileEntry> > createEntries() {
   SharedHandle<FileEntry> array[] = {
     SharedHandle<FileEntry>(new FileEntry("file0.txt", 0, 0)),
     SharedHandle<FileEntry>(new FileEntry("file1.txt", 15, 0)),
@@ -68,9 +68,8 @@ std::deque<SharedHandle<FileEntry> > createEntries() {
   //                         *** file6
   //                            |file7
   //                            ** file8
-  std::deque<SharedHandle<FileEntry> > entries(&array[0],
-					       &array[arrayLength(array)]);
-  for(std::deque<SharedHandle<FileEntry> >::const_iterator i = entries.begin();
+  std::vector<SharedHandle<FileEntry> > entries(vbegin(array), vend(array));
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator i = entries.begin();
       i != entries.end(); ++i) {
     File((*i)->getPath()).remove();
   }
@@ -80,12 +79,12 @@ std::deque<SharedHandle<FileEntry> > createEntries() {
 void MultiDiskAdaptorTest::testResetDiskWriterEntries()
 {
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
-    adaptor->setFileEntries(fileEntries);
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -97,13 +96,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     fileEntries[0]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     // Because entries[1] spans entries[0]
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
@@ -116,14 +115,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     fileEntries[0]->setRequested(false);
     fileEntries[1]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
     // Because entries[2] spans entries[1]
@@ -137,13 +136,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     fileEntries[3]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -157,13 +156,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     fileEntries[4]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -176,14 +175,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     fileEntries[3]->setRequested(false);
     fileEntries[4]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -195,15 +194,15 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     for(size_t i = 5; i < 9; ++i) {
       fileEntries[i]->setRequested(false);
     }
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     // In openFile(), resetDiskWriterEntries() are called.
     adaptor->openFile();
     
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -215,13 +214,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     for(size_t i = 1; i < 9; ++i) {
       fileEntries[i]->setRequested(false);
     }
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     adaptor->openFile();
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
@@ -233,13 +232,13 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     for(size_t i = 2; i < 9; ++i) {
       fileEntries[i]->setRequested(false);
     }
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     adaptor->openFile();
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(!entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(!entries[1]->getDiskWriter().isNull());
@@ -253,14 +252,14 @@ void MultiDiskAdaptorTest::testResetDiskWriterEntries()
     adaptor->closeFile();
   }
   {
-    std::deque<SharedHandle<FileEntry> > fileEntries = createEntries();
+    std::vector<SharedHandle<FileEntry> > fileEntries = createEntries();
     for(size_t i = 0; i < 6; ++i) {
       fileEntries[i]->setRequested(false);
     }
     fileEntries[8]->setRequested(false);
-    adaptor->setFileEntries(fileEntries);
+    adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
     adaptor->openFile();
-    std::deque<SharedHandle<DiskWriterEntry> > entries =
+    std::vector<SharedHandle<DiskWriterEntry> > entries =
       adaptor->getDiskWriterEntries();
     CPPUNIT_ASSERT(entries[0]->getDiskWriter().isNull());
     CPPUNIT_ASSERT(entries[1]->getDiskWriter().isNull());
@@ -290,8 +289,8 @@ void readFile(const std::string& filename, char* buf, int bufLength) {
 }
 
 void MultiDiskAdaptorTest::testWriteData() {
-  try {
-  adaptor->setFileEntries(createEntries());
+  std::vector<SharedHandle<FileEntry> > fileEntries(createEntries());
+  adaptor->setFileEntries(fileEntries.begin(), fileEntries.end());
 
   adaptor->openFile();
   std::string msg = "12345";
@@ -335,22 +334,18 @@ void MultiDiskAdaptorTest::testWriteData() {
   CPPUNIT_ASSERT_EQUAL(std::string("12"), std::string(buf));
 
   CPPUNIT_ASSERT(File("file5.txt").isFile());
-
-  } catch(Exception& e) {
-    CPPUNIT_FAIL(e.stackTrace());
-  }
 }
 
 void MultiDiskAdaptorTest::testReadData() {
   SharedHandle<FileEntry> entry1(new FileEntry("file1r.txt", 15, 0));
   SharedHandle<FileEntry> entry2(new FileEntry("file2r.txt", 7, 15));
   SharedHandle<FileEntry> entry3(new FileEntry("file3r.txt", 3, 22));
-  std::deque<SharedHandle<FileEntry> > entries;
+  std::vector<SharedHandle<FileEntry> > entries;
   entries.push_back(entry1);
   entries.push_back(entry2);
   entries.push_back(entry3);
 
-  adaptor->setFileEntries(entries);
+  adaptor->setFileEntries(entries.begin(), entries.end());
 
   adaptor->openFile();
   unsigned char buf[128];
@@ -370,20 +365,20 @@ void MultiDiskAdaptorTest::testReadData() {
 
 void MultiDiskAdaptorTest::testCutTrailingGarbage()
 {
-  std::string dir = "/tmp";
+  std::string dir = "./";
   std::string prefix = "aria2_MultiDiskAdaptorTest_testCutTrailingGarbage_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 256, 0)),
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 512, 256))
   };
-  for(size_t i = 0; i < arrayLength(entries); ++i) {
+  for(size_t i = 0; i < A2_ARRAY_LEN(entries); ++i) {
     createFile(entries[i]->getPath(), entries[i]->getLength()+100);
   }
-  std::deque<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
+  std::vector<SharedHandle<FileEntry> > fileEntries
+    (vbegin(entries), vend(entries));
   
   MultiDiskAdaptor adaptor;
-  adaptor.setFileEntries(fileEntries);
+  adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
   
@@ -392,27 +387,27 @@ void MultiDiskAdaptorTest::testCutTrailingGarbage()
   adaptor.cutTrailingGarbage();
 
   CPPUNIT_ASSERT_EQUAL((uint64_t)256,
-		       File(entries[0]->getPath()).size());
+                       File(entries[0]->getPath()).size());
   CPPUNIT_ASSERT_EQUAL((uint64_t)512,
-		       File(entries[1]->getPath()).size());
+                       File(entries[1]->getPath()).size());
 }
 
 void MultiDiskAdaptorTest::testSize()
 {
-  std::string dir = "/tmp";
+  std::string dir = "./";
   std::string prefix = "aria2_MultiDiskAdaptorTest_testSize_";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"1", 1, 0)),
     SharedHandle<FileEntry>(new FileEntry(dir+"/"+prefix+"2", 1, 1))
   };
-  for(size_t i = 0; i < arrayLength(entries); ++i) {
+  for(size_t i = 0; i < A2_ARRAY_LEN(entries); ++i) {
     createFile(entries[i]->getPath(), entries[i]->getLength());
   }
-  std::deque<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
+  std::vector<SharedHandle<FileEntry> > fileEntries
+    (vbegin(entries), vend(entries));
   
   MultiDiskAdaptor adaptor;
-  adaptor.setFileEntries(fileEntries);
+  adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
   adaptor.setMaxOpenFiles(1);
   adaptor.setPieceLength(1);
   
@@ -423,7 +418,7 @@ void MultiDiskAdaptorTest::testSize()
 
 void MultiDiskAdaptorTest::testUtime()
 {
-  std::string storeDir = "/tmp/aria2_MultiDiskAdaptorTest_testUtime";
+  std::string storeDir = "./aria2_MultiDiskAdaptorTest_testUtime";
   SharedHandle<FileEntry> entries[] = {
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/requested", 0, 0)),
     SharedHandle<FileEntry>(new FileEntry(storeDir+"/notFound", 0, 0)),
@@ -438,10 +433,10 @@ void MultiDiskAdaptorTest::testUtime()
 
   entries[2]->setRequested(false);
 
-  std::deque<SharedHandle<FileEntry> > fileEntries
-    (&entries[0], &entries[arrayLength(entries)]);
+  std::vector<SharedHandle<FileEntry> > fileEntries
+    (vbegin(entries), vend(entries));
   MultiDiskAdaptor adaptor;
-  adaptor.setFileEntries(fileEntries);
+  adaptor.setFileEntries(fileEntries.begin(), fileEntries.end());
 
   time_t atime = (time_t) 100000;
   time_t mtime = (time_t) 200000;
@@ -449,13 +444,13 @@ void MultiDiskAdaptorTest::testUtime()
   CPPUNIT_ASSERT_EQUAL((size_t)2, adaptor.utime(Time(atime), Time(mtime)));
   
   CPPUNIT_ASSERT_EQUAL((time_t)mtime,
-		       File(entries[0]->getPath()).getModifiedTime().getTime());
+                       File(entries[0]->getPath()).getModifiedTime().getTime());
 
   CPPUNIT_ASSERT_EQUAL((time_t)mtime,
-		       File(entries[3]->getPath()).getModifiedTime().getTime());
+                       File(entries[3]->getPath()).getModifiedTime().getTime());
 
   CPPUNIT_ASSERT((time_t)mtime !=
-		 File(entries[2]->getPath()).getModifiedTime().getTime());
+                 File(entries[2]->getPath()).getModifiedTime().getTime());
 }
 
 } // namespace aria2
